package com.syjy.calculate.entity;

import java.math.BigDecimal;
import java.util.HashMap;

/**
 * 数据返回格式化
 *
 * @author zcl
 * @version 1.0
 * @since 2022/9/9 10:34
 */
public class CalculateResult extends HashMap<String, Object> {

    private static final long serialVersionUID = 1L;

    /** 100 */
    public static final BigDecimal ONE_HUNDRED = new BigDecimal("100");

    /** 百分号 */
    public static final String PERCENT = "%";

    /** formula */
    public static final String FORMULA = "FORMULA";

    /** rules */
    public static final String RULES = "RULES";

    /** 下划线 */
    public static final String UNDERLINE = "_";

    /** 忽略 */
    public static final String IGNORE = "666";

    /** 失败 */
    public static final String ERROR = "500";

    /** 状态码 */
    public static final String CODE_TAG = "code";

    /** 返回内容 */
    public static final String MSG_TAG = "msg";

    /** 数据对象 */
    public static final String DATA_TAG = "data";

    /** 公式上下文中的数据 */
    public static final String LIST = "list";

    /** 场站标识 */
    public static final String SIGN = "sign";

    /** 容量 */
    public static final String CAPACITY = "electricCapacity";

    /** 时间 */
    public static final String TIME = "time";

    /** 消息信息*/
    public static final String MSG_REQUEST_NULL = "入参为空";
    public static final String MSG_FORMULA_TYPE_NULL = "公式类型为空";
    public static final String MSG_PROVINCE_NULL = "省调为空";
    public static final String MSG_ELECTRIC_TYPE_NULL = "场站类型为空";
    public static final String MSG_CALCULATE_DATA_NULL = "计算数据为空";
    public static final String MSG_USEFUL_CALCULATE_DATA_NULL = "可用的计算数据为空";
    public static final String MSG_CAP_NULL = "容量为空/0，不予考核";
    public static final String MSG_DATA_CHECK_ERROR = "数据校验错误";
    public static final String MSG_NO_FORMULA = "未匹配到公式:";
    public static final String MSG_CALCULATE_SUCCESS = "计算成功";
    public static final String MSG_CALCULATE_FAIL = "计算失败:";
    public static final String MSG_CAP_ZERO = "容量为0";
    public static final String MSG_SIZE_ZERO = "个数为0";

    /**
     * 状态类型
     */
    public enum Type
    {
        /** 成功 */
        SUCCESS("200"),
        /** 忽略 */
        IGNORE("666"),
        /** 警告 */
        WARN("301"),
        /** 错误 */
        ERROR("500");
        private final String value;

        Type(String value)
        {
            this.value = value;
        }

        public String value()
        {
            return this.value;
        }
    }

    /**
     * 初始化一个新创建的 CalculateResult 对象
     *
     * @param type 状态类型
     * @param msg 返回内容
     */
    public CalculateResult(Type type, String msg)
    {
        super.put(CODE_TAG, type.value);
        super.put(MSG_TAG, msg);
    }

    /**
     * 初始化一个新创建的 CalculateResult 对象
     *
     * @param type 状态类型
     * @param msg 返回内容
     * @param data 数据对象
     */
    public CalculateResult(Type type, String msg, Object data)
    {
        super.put(CODE_TAG, type.value);
        super.put(MSG_TAG, msg);
        if (data != null) {
            super.put(DATA_TAG, data);
        }
    }

    /**
     * 返回成功消息
     *
     * @return 成功消息
     */
    public static CalculateResult success()
    {
        return CalculateResult.success("操作成功");
    }

    /**
     * 返回成功数据
     *
     * @return 成功消息
     */
    public static CalculateResult success(Object data)
    {
        return CalculateResult.success("操作成功", data);
    }

    /**
     * 返回成功消息
     *
     * @param msg 返回内容
     * @return 成功消息
     */
    public static CalculateResult success(String msg)
    {
        return CalculateResult.success(msg, null);
    }

    /**
     * 返回成功消息
     *
     * @param msg 返回内容
     * @param data 数据对象
     * @return 成功消息
     */
    public static CalculateResult success(String msg, Object data)
    {
        return new CalculateResult(Type.SUCCESS, msg, data);
    }

    /**
     * 返回警告消息
     *
     * @param msg 返回内容
     * @return 警告消息
     */
    public static CalculateResult warn(String msg)
    {
        return CalculateResult.warn(msg, null);
    }

    /**
     * 返回警告消息
     *
     * @param msg 返回内容
     * @param data 数据对象
     * @return 警告消息
     */
    public static CalculateResult warn(String msg, Object data)
    {
        return new CalculateResult(Type.WARN, msg, data);
    }

    /**
     * 返回错误消息
     *
     * @return
     */
    public static CalculateResult error()
    {
        return CalculateResult.error("操作失败");
    }

    /**
     * 返回错误消息
     *
     * @param msg 返回内容
     * @return 警告消息
     */
    public static CalculateResult error(String msg)
    {
        return CalculateResult.error(msg, null);
    }

    /**
     * 返回错误消息
     *
     * @param msg 返回内容
     * @param data 数据对象
     * @return 警告消息
     */
    public static CalculateResult error(String msg, Object data)
    {
        return new CalculateResult(Type.ERROR, msg, data);
    }
}

